package de.jottyfan.timetrack.modules;

import static de.jottyfan.timetrack.db.profile.Tables.T_LOGIN;
import static de.jottyfan.timetrack.db.profile.Tables.V_LOGINROLE;

import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.CloseableDSLContext;
import org.jooq.Record1;
import org.jooq.Record4;
import org.jooq.SelectConditionStep;
import org.jooq.UpdateConditionStep;
import org.jooq.exception.DataAccessException;

import de.jooqfaces.JooqFacesContext;
import de.jottyfan.timetrack.db.profile.tables.records.TLoginRecord;

/**
 * 
 * @author henkej
 *
 */
public class SessionGateway extends JooqGateway {
	private static final Logger LOGGER = LogManager.getLogger();

	public SessionGateway(JooqFacesContext facesContext) {
		super(facesContext);
	}

	/**
	 * seek login and set bean's login to that; if not found, return false
	 * 
	 * @param bean
	 *          the bean
	 * @return true or false
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * @throws DataAccessException
	 */
	public boolean seekAndSetLogin(SessionBean bean) throws DataAccessException, ClassNotFoundException, SQLException {
		try (CloseableDSLContext jooq = getJooq()) {
			SelectConditionStep<Record4<Integer, String, String, String>> sql = jooq
			// @formatter:off
				.select(T_LOGIN.PK,
						    T_LOGIN.FORENAME,
						    T_LOGIN.SURNAME,
						    T_LOGIN.PASSWORD)
				.from(T_LOGIN)
				.where(T_LOGIN.LOGIN.eq(bean.getUsername()));
			// @formatter:on
			LOGGER.debug(sql.toString());
			Record4<Integer, String, String, String> r = sql.fetchOne();
			if (r != null) {
				String encrypted = r.get(T_LOGIN.PASSWORD);
				if (bean.checkSecret(encrypted)) {
					bean.setLogin(r.get(T_LOGIN.PK));
					bean.setForename(r.get(T_LOGIN.FORENAME));
					bean.setSurname(r.get(T_LOGIN.SURNAME));

					SelectConditionStep<Record1<String>> sql2 = jooq
					// @formatter:off
						.select(V_LOGINROLE.ROLE_NAME)
						.from(V_LOGINROLE)
						.where(V_LOGINROLE.LOGIN.eq(bean.getUsername()));
					// @formatter:on
					LOGGER.debug(sql2.toString());
					for (Record1<String> privilege : sql2.fetch()) {
						bean.getPrivileges().add(privilege.get(V_LOGINROLE.ROLE_NAME));
					}
					addToSessionMap("sessionBean", bean);
					return true;
				} else {
					throw new DataAccessException("wrong password");
				}
			} else {
				return false;
			}
		}
	}

	/**
	 * change the password
	 * 
	 * @param bean
	 *          the bean containing the new password
	 * @return true or false
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * @throws DataAccessException
	 */
	public boolean changePassword(SessionBean bean, String newPassword)
			throws DataAccessException, ClassNotFoundException, SQLException {
		bean.setSecret(newPassword);
		String encryptedPassword = bean.getSecret();
		try (CloseableDSLContext jooq = getJooq()) {
			UpdateConditionStep<TLoginRecord> sql = jooq
			// @formatter:off
					.update(T_LOGIN)
					.set(T_LOGIN.PASSWORD, encryptedPassword)
					.where(T_LOGIN.PK.eq(bean.getLogin()));
				// @formatter:on
			LOGGER.debug("updating password, disable log here");
			return sql.execute() == 1;
		}
	}
}
